@@ -3,9 +3,6 @@
 
 
 struct sPilotskinConfig {
-  int red;
-  int green;
-  int blue;
   int closeonswitch;
   int hidemenu;
 };
