@@ -347,15 +347,19 @@ bool cLogChain::Parse(const unsigned char *cat)
 class cLogger : public cAction {
 private:
   int cardNum;
-  bool softCSA, up, prescan;
+  bool softCSA, up;
   cSimpleList<cLogChain> chains;
   cSimpleList<cEcmInfo> active;
   //
   cPidFilter *catfilt;
   int catVers;
   //
+  enum ePreMode { pmNone, pmStart, pmWait, pmActive, pmStop };
+  ePreMode prescan;
+  cTimeMs pretime;
+  //
   cPidFilter *AddFilter(int Pid, int Section, int Mask, int Mode, int IdleTime, bool Crc);
-  void SetChains(bool forcestop=false);
+  void SetChains(void);
   void ClearChains(void);
   void StartChain(cLogChain *chain);
   void StopChain(cLogChain *chain, bool force);
@@ -374,7 +378,7 @@ cLogger::cLogger(int CardNum, bool soft)
 :cAction("logger",CardNum)
 {
   cardNum=CardNum; softCSA=soft;
-  catfilt=0; up=prescan=false;
+  catfilt=0; up=false; prescan=pmNone;
   Priority(10);
 }
 
@@ -402,7 +406,7 @@ void cLogger::Down(void)
     PRINTF(L_CORE_AUEXTRA,"%d: DOWN",cardNum);
     ClearChains();
     DelAllFilter();
-    catfilt=0; up=prescan=false;
+    catfilt=0; up=false; prescan=pmNone;
     }
   Unlock();
 }
@@ -410,7 +414,7 @@ void cLogger::Down(void)
 void cLogger::PreScan(void)
 {
   Lock();
-  prescan=true; Up(); SetChains();
+  prescan=pmStart; Up();
   Unlock();
 }
 
@@ -431,16 +435,17 @@ void cLogger::EcmStatus(const cEcmInfo *ecm, bool on)
         break;
         }
     }
-  bool presave=prescan; prescan=false;
-  SetChains(presave);
+  if(prescan>=pmWait) prescan=pmStop;
+  SetChains();
+  prescan=pmNone;
   Unlock();
 }
 
-void cLogger::SetChains(bool forcestop)
+void cLogger::SetChains(void)
 {
   for(cLogChain *chain=chains.First(); chain; chain=chains.Next(chain)) {
     bool act=false;
-    if(ScSetup.AutoUpdate>1 || prescan) act=true;
+    if(ScSetup.AutoUpdate>1 || prescan==pmActive) act=true;
     else if(ScSetup.AutoUpdate==1) {
       for(cEcmInfo *e=active.First(); e; e=active.Next(e))
         if((e->emmCaId && chain->caid==e->emmCaId) || chain->caid==e->caId) {
@@ -448,7 +453,7 @@ void cLogger::SetChains(bool forcestop)
           }
        }
     if(act) StartChain(chain);
-    else StopChain(chain,forcestop);
+    else StopChain(chain,prescan==pmStop);
     }
 }
 
@@ -539,7 +544,9 @@ void cLogger::Process(cPidFilter *filter, unsigned char *data, int len)
             }
           }
         SetChains();
+        if(prescan==pmStart) { prescan=pmWait; pretime.Set(2000); }
         }
+      if(prescan==pmWait && pretime.TimedOut()) { prescan=pmActive; SetChains(); }
       }
     else {
       HEXDUMP(L_HEX_EMM,data,len,"EMM pid 0x%04x",filter->Pid());
@@ -1366,7 +1373,7 @@ void cCam::Tune(const cChannel *channel)
 
 void cCam::PostTune(void)
 {
-  if(true) { //XXX configurable??
+  if(ScSetup.PrestartAU) {
     LogStartup();
     if(logger) logger->PreScan();
     }
@@ -1456,12 +1463,10 @@ void cCam::HouseKeeping(void)
     if(handler->IsRemoveable()) RemHandler(handler);
     handler=next;
     }
-/* XXX
-  if(handlerList.Count()<1) {
+  if(handlerList.Count()<1 && !ScSetup.PrestartAU) {
     delete hookman; hookman=0;
     delete logger; logger=0;
     }
-*/
 }
 
 bool cCam::GetPrgCaids(int source, int transponder, int prg, caid_t *c)
@@ -2642,8 +2647,13 @@ void cScDvbDevice::Capture(void)
   objdump -T <path-to-vdr>/vdr | grep -E "(useDevice|nextCardIndex)"
   Insert the symbol names below.
 */
+#if __GNUC__ >= 3
   vdr_nci=(int *)dlsym(RTLD_DEFAULT,"_ZN7cDevice13nextCardIndexE");
   vdr_ud =(int *)dlsym(RTLD_DEFAULT,"_ZN7cDevice9useDeviceE");
+#else
+  vdr_nci=(int *)dlsym(RTLD_DEFAULT,"_7cDevice.nextCardIndex");
+  vdr_ud =(int *)dlsym(RTLD_DEFAULT,"_7cDevice.useDevice");
+#endif
   if(vdr_nci && vdr_ud) { vdr_save_ud=*vdr_ud; *vdr_ud=1<<30; }
 }
 
