@@ -927,10 +927,6 @@ int c6805::Run(int max_count)
         case 0xF:
         case 0x0:			// bit
         case 0x1:
-	  if (ea==0x16) { 
-	    unsigned char old=Get(ea); 
-	    if(old&2) op=(old&~0x02) | (op&0x02); 
-	  }
           Set(ea,op); break;
         case 0x4:			// inherent A
           a=op; break;
