@@ -24,13 +24,17 @@
 #include <libsi/section.h>
 
 #include "viaccess.h"
+#include "log-viaccess.h"
 #include "tps.h"
+#include "opentv.h"
+#include "st20.h"
+
 #include "sc.h"
 #include "scsetup.h"
 #include "system-common.h"
 #include "filter.h"
 #include "misc.h"
-#include "log-viaccess.h"
+#include "helper.h"
 
 #define CHECK_TIME    5*60*1000
 #define LOADBIN_TIME 60*60*1000
@@ -258,6 +262,240 @@ void cSatTime::CheckHandler(void)
     }
 }
 
+// -- cOpenTVModule ------------------------------------------------------------
+
+class cOpenTVModule {
+private:
+  int id, modlen;
+  unsigned char *mem;
+  int received;
+  info_header_t info;
+  code_header_t code;
+  data_header_t data;
+  //
+  bool Decompress(void);
+  bool DoDecompress(unsigned char *out_ptr, const unsigned char *in_ptr, int dsize, int data_size, int usize);
+  bool ParseSections(void);
+  void Dump(void);
+public:
+  cOpenTVModule(const unsigned char *data);
+  ~cOpenTVModule();
+  int AddPart(const unsigned char *data, int len);
+  const info_header_t *InfoHdr(void) const { return &info; }
+  const code_header_t *CodeHdr(void) const { return &code; }
+  const data_header_t *DataHdr(void) const { return &data; }
+  //
+  inline static int Id(const unsigned char *data) { return UINT16_BE(&data[3]); }
+  inline static int Offset(const unsigned char *data) { return UINT32_BE(&data[16]); }
+  inline static int Length(const unsigned char *data) { return UINT32_BE(&data[20]); }
+  };
+
+cOpenTVModule::cOpenTVModule(const unsigned char *data)
+{
+  id=Id(data); modlen=Length(data);
+  received=0;
+  mem=MALLOC(unsigned char,modlen);
+}
+
+cOpenTVModule::~cOpenTVModule()
+{
+  free(mem);
+}
+
+int cOpenTVModule::AddPart(const unsigned char *data, int slen)
+{
+  if(Id(data)==id) {
+    int off=Offset(data);
+    int mlen=Length(data);
+    int rec=slen-28;
+    if(mlen!=modlen || (off+rec)>mlen || !mem) {
+      PRINTF(L_SYS_TPSAU,"length mismatch while adding to OpenTV module");
+      return -1;
+      }
+    memcpy(&mem[off],data+24,rec);
+    received+=rec;
+    if(received==mlen) {
+      if(!Decompress()) {
+        PRINTF(L_SYS_TPSAU,"failed to decompress OpenTV module");
+        return -1;
+        }
+      if(!ParseSections()) {
+        PRINTF(L_SYS_TPSAU,"DATA & CODE section not located in OpenTV module");
+        return -1;
+        }
+      Dump();
+      return 1;
+      }
+    }
+  return 0;
+}
+
+void cOpenTVModule::Dump(void)
+{
+#ifdef DUMP_TPSAU
+#warning Dumping TPS AU data
+  {
+  char fname[32];
+  snprintf(fname,sizeof(fname),"%s/decomp.bin.%d.%x",DUMP_TPSAU,getpid(),(int)time(0));
+  int fd=open(fname,O_CREAT|O_TRUNC|O_WRONLY,DEFFILEMODE);
+  if(fd>=0) {
+    write(fd,mem,modlen);
+    close(fd);
+    PRINTF(L_SYS_TPSAU,"dumped to file '%s'",fname);
+    }
+  }
+#endif
+}
+
+bool cOpenTVModule::ParseSections(void)
+{
+  int sections=0;
+  for(int idx=0; idx<modlen;) {
+    unsigned int hdr=UINT32_BE(mem+idx);
+    unsigned int s_len=UINT32_BE(mem+idx+4);
+    switch(hdr) {
+      case INFO_SECTION_HDR:
+	info.magic=hdr;
+	info.bsssize=UINT32_BE(mem+idx+8);
+	info.stacksize=UINT32_BE(mem+idx+12);
+        sections|=1;
+        break;
+      case CODE_SECTION_HDR:
+	code.magic=hdr;
+	code.size=s_len-8;
+	code.m_id=UINT32_BE(mem+idx+8);
+	code.entry_point=UINT16_BE(mem+idx+12);
+	code.end=UINT16_BE(mem+idx+14);
+	code.code=mem+idx+8;
+        sections|=2;
+        break;
+      case DATA_SECTION_HDR:
+	data.magic=hdr;
+	data.dlen=s_len-8;
+	data.data=mem+idx+8;
+        sections|=4;
+        break;
+      case SWAP_SECTION_HDR:
+      case GDBO_SECTION_HDR:
+        break;
+      case LAST_SECTION_HDR:
+      case 0:
+        idx=modlen;
+        break;
+      case COMP_SECTION_HDR:
+        PRINTF(L_GEN_DEBUG,"OpenTV module still compressed in ParseSections()");
+        return 0;
+      default:
+        PRINTF(L_SYS_TPSAU,"unknown section hdr %08x (%c%c%c%c) in OpenTV module",hdr,(hdr>>24)&0xFF,(hdr>>16)&0xFF,(hdr>>8)&0xFF,hdr&0xFF);
+        break;
+      }
+    idx+=s_len;
+    }
+  return sections>=6;
+}
+
+bool cOpenTVModule::Decompress(void)
+{
+  comp_header_t comp;
+  comp.magic=UINT32_BE(mem);
+  comp.csize=UINT32_BE(mem+4);
+  comp.dsize=UINT32_BE(mem+8);
+  comp.usize=UINT32_BE(mem+12);
+  comp.end  =UINT8_BE(mem+16);
+  if((COMP_SECTION_HDR!=comp.magic) || (comp.dsize<=comp.csize) || 
+     (comp.usize>comp.dsize) || (comp.end>=1) || (comp.csize<=17))
+    return true;
+  unsigned char *decomp=MALLOC(unsigned char,comp.dsize);
+  if(!decomp || !DoDecompress(decomp,mem+17,comp.dsize,comp.csize-17,comp.usize))
+    return false;
+  free(mem);
+  mem=decomp; modlen=comp.dsize;
+  return true;
+}
+
+#define BYTE() ((odd)?((in_ptr[0]&0x0F)|(in_ptr[1]&0xF0)):in_ptr[0])
+
+#define NIBBLE(__v) \
+  do { \
+    odd^=1; \
+    if(odd) __v=in_ptr[0]&0xF0; \
+    else {  __v=(in_ptr[0]&0xF)<<4; in_ptr++; } \
+    } while(0)
+
+bool cOpenTVModule::DoDecompress(unsigned char *out_ptr, const unsigned char *in_ptr, int dsize, int data_size, int usize)
+{
+  if(usize==0) return false;
+  const unsigned char *data_start=in_ptr;
+  unsigned char *out_start=out_ptr;
+  unsigned char *out_end=out_ptr+usize;
+  int odd=0;
+  while(1) {
+    unsigned char mask=BYTE(); in_ptr++;
+    for(int cnt=8; cnt>0; mask<<=1,cnt--) {
+      if(mask&0x80) {
+        out_ptr[0]=BYTE(); in_ptr++;
+        out_ptr++;
+        }
+      else {
+        int off=0, len=0;
+        unsigned char cmd=BYTE(); in_ptr++;
+        switch(cmd>>4) {
+          case 0x0 ... 0x6:
+            off=((cmd&0xF)<<8)+BYTE(); in_ptr++;
+            len=((cmd>>4)&0x7)+3;
+            break;
+          case 0x7:
+            {
+            unsigned char high=BYTE(); in_ptr++;
+            off=((high&0x7F)<<8)+BYTE(); in_ptr++;
+            if((cmd==0x7F) && (high&0x80)) {
+              len=BYTE(); in_ptr++;
+              if(len==0xFF) {
+                len=BYTE(); in_ptr++;
+                len=((len<<8)+BYTE()+0x121)&0xFFFF; in_ptr++;
+                }
+              else len+=0x22;
+              }
+            else {
+              len=((cmd&0x0F)<<1)+3; if(high&0x80) len++;
+              }
+            break;
+            }
+          case 0x8 ... 0xB:
+            if(cmd&0x20) NIBBLE(off); else off=0;
+            off=(off<<1)|(cmd&0x1F); len=2;
+            break;
+          case 0xC:
+            off=cmd&0x0F; len=3;
+            break;
+          case 0xD ... 0xF:
+            NIBBLE(off);
+            off|=cmd&0x0F; len=((cmd>>4)&0x3)+2;
+            break;
+          }
+        const unsigned char *from=out_ptr-(off+1);
+        if(from<out_start || from>=out_ptr || len>(out_end-out_ptr)) {
+          PRINTF(L_SYS_TPSAU,"length mismatch in OpenTV decompress");
+          return false;
+          }
+        while(--len>=0) *out_ptr++=*from++;
+        }
+      if(out_end<=out_ptr) {
+        if(out_end!=out_ptr) {
+          PRINTF(L_SYS_TPSAU,"pointer mismatch in OpenTV decompress");
+          return false;
+          }
+        int len=out_start+dsize-out_ptr;
+        if(len>0) memmove(out_ptr,data_start+(data_size-(dsize-usize)),len);
+        return true;
+        }
+      }
+    }
+}
+
+#undef BYTE
+#undef NIBBLE
+
 // -- cTpsKey ------------------------------------------------------------------
 
 cTpsKey::cTpsKey(void)
@@ -298,8 +536,7 @@ void cTpsKey::Put(unsigned char *mem) const
 
 class cTpsAuHook : public cLogHook {
 private:
-  unsigned char *mem;
-  int pos;
+  cOpenTVModule *mod;
   int pmtpid, aupid;
 public:
   cTpsAuHook(void);
@@ -317,12 +554,13 @@ cTpsKeys::cTpsKeys(void)
 ,lastAu(-TPSAU_TIME)
 {
   list=new cSimpleList<cTpsKey>;
-  first=last=0;
+  first=last=0; algomem=0;
 }
 
 cTpsKeys::~cTpsKeys()
 {
   delete list;
+  free(algomem);
 }
 
 const cTpsKey *cTpsKeys::GetKey(time_t t)
@@ -333,6 +571,25 @@ const cTpsKey *cTpsKeys::GetKey(time_t t)
   return 0;
 }
 
+const cTpsKey *cTpsKeys::GetV2Key(int id)
+{
+  unsigned char tmp[56];
+  memset(tmp,0,sizeof(tmp));
+  cPlainKey *pk=keys.FindKey('V',id,MBC3('T','P','S'),16,0);
+  if(pk) {
+    pk->Get(&tmp[4+2*16]);
+    tmp[52+2]=1;
+    tmp[52+3]=0x1C;
+    cTpsKey *tk=new cTpsKey;
+    if(tk) {
+      tk->Set(tmp);
+      return tk;
+      }
+    }
+  else PRINTF(L_SYS_KEY,"missing %.4x TPS key\n",id);
+  return 0;
+}
+
 void cTpsKeys::Check(time_t now, int cardnum)
 {
   checkMutex.Lock();
@@ -343,12 +600,14 @@ void cTpsKeys::Check(time_t now, int cardnum)
     lastCheck.Set();
     }
   bool nokey=now+2*3600>last;
+/*
   if(lastLoad.Elapsed()>(nokey ? LOADBIN_TIME/60 : LOADBIN_TIME)) {
     PRINTF(L_SYS_TPSAU,"loading "TPSBIN" triggered");
     LoadBin();
     if(now>0) Purge(now);
     lastLoad.Set();
     }
+*/
   if(lastAu.Elapsed()>(nokey ? TPSAU_TIME/60 : TPSAU_TIME)) {
     if(ScSetup.AutoUpdate>0) {
       PRINTF(L_SYS_TPSAU,"TPS AU triggered");
@@ -439,46 +698,60 @@ void cTpsKeys::GetFirstLast(void)
     }
 }
 
-bool cTpsKeys::ProcessAu(unsigned char *mem, int len)
+bool cTpsKeys::ProcessAu(const cOpenTVModule *mod)
 {
-  PRINTF(L_SYS_TPSAU,"processing TPS AU data (len=%d)",len);
-#ifdef DUMP_TPSAU
-#warning Dumping TPS AU data
-  {
-  char fname[32];
-  snprintf(fname,sizeof(fname),"%s/tpsau.dump.%d.%x",DUMP_TPSAU,getpid(),(int)time(0));
-  int fd=open(fname,O_CREAT|O_TRUNC|O_WRONLY);
-  if(fd>=0) {
-    write(fd,mem,len);
-    close(fd);
-    PRINTF(L_SYS_TPSAU,"dumped to file '%s'",fname);
+  PRINTF(L_SYS_TPSAU,"processing TPS AU data");
+
+  const code_header_t *codehdr=mod->CodeHdr();
+  const data_header_t *datahdr=mod->DataHdr();
+  const unsigned char *c=codehdr->code;
+  const unsigned char *d=datahdr->data;
+  unsigned int kd=0, cb1=0, cb2=0, cb3=0;
+  for(unsigned int i=0; i<codehdr->size; i++) {
+    if(c[i] == 0x81) { // PushEA DS:$xxxx
+      unsigned int addr=(c[i+1]<<8)|c[i+2];
+      if(addr<(datahdr->dlen-3)) {
+        if(d[addr+1]==0x00 && d[addr+3]==0x00 && d[addr+4]==3) kd=addr;
+        else if(d[addr]==0x73 && d[addr+1]==0x25) {
+          static const unsigned char scan1[] = { 0x28, 0x20, 0x20, 0xC0 };
+          for(int j=2; j < 0xC; j++)
+            if(!memcmp(&d[addr+j],scan1,sizeof(scan1))) { cb1=addr; break; }
+          }
+        else if((d[addr]&0xF0)==0x60 && (d[addr+1]&0xF0)==0xB0) {
+          int vajw = (int)(((~(d[addr]&0x0F))<<4)|(d[addr+1]&0x0F));
+          unsigned char hits=0;
+          for(int j=2; j < 0x30; j++) {
+            int vld = ((d[addr+j]&0x0F)<<4)|(d[addr+j+1]&0x0F);
+            if((d[addr+j]&0xF0)==0x20 && (d[addr+j+1]&0xF0)==0x70) {
+              int val=vajw+vld;
+              if(val==3 || val==4 || val==5) hits++;
+              }
+            }
+          if(hits==3) cb3=addr;
+          else if(hits==2) {
+            if(cb2==0) cb2=addr;
+            else if(cb3==0) cb3=addr;
+            }
+          }
+        }
+      }
     }
-  }
-#endif
-  unsigned char *data;
+  if(!kd || !cb1 || !cb2 || !cb3) {
+    PRINTF(L_SYS_TPSAU,"couldn't locate all pointers in data section");
+    return false;
+    }
+  RegisterAlgo3(d,cb1,cb2,cb3,kd);
+
+  const unsigned char *data=&d[kd];
   int seclen, numkeys;
-  for(int i=0; ; i++) {
-    static const unsigned char startcode[] = {
-      0x6F,0x61,0x64,0x63,0x61,0x73,0x74,0x2E,0x63,0x68,0x61,0x6E,0x6E,0x65,0x6C,0x2E,
-      0x31,0x33,0x35,0x2E,0x34,0x35,0x31,0x09,0x05,0x4F,0x00,0x00,0x00,0x00,0x00 };
-    if(memcmp(&mem[i],startcode,sizeof(startcode))==0) {
-      data=&mem[i+32];
-      seclen=data[0] | (data[1]<<8);
-      numkeys=data[2] | (data[3]<<8);
-      if(seclen>4 && numkeys>0) break;
-      }
-    if(i>len-200) {
-      PRINTF(L_SYS_TPSAU,"no startcode found");
-      return false;
-      }
-    }
+  seclen=data[0] | (data[1]<<8);
+  numkeys=data[2] | (data[3]<<8);
   int algo=data[4];
   int mkidx=data[5]&7;
   unsigned char *sec[7];
-  sec[0]=data+6;
+  sec[0]=(unsigned char *)data+6;
   for(int i=1; i<6; i++) sec[i]=sec[i-1]+seclen;
   sec[6]=sec[5]+numkeys;
-  PRINTF(L_SYS_TPSAU,"found startcode (off=%d seclen=%d numkeys=%d keyoff=%d algo=%d mkidx=%d)",data-mem,seclen,numkeys,sec[6]-mem,algo,mkidx);
   unsigned char key[16];
   cPlainKey *pk=keys.FindKey('V',0x7c00,MBC3('M','K',mkidx),16);
   if(!pk) {
@@ -486,14 +759,15 @@ bool cTpsKeys::ProcessAu(unsigned char *mem, int len)
     return false;
     }
   pk->Get(key);
-  if(sec[6]>=mem+len) {
+
+  if(sec[6]>=d+datahdr->dlen) {
     PRINTF(L_SYS_TPSAU,"section 5 exceeds buffer");
     return false;
     }
   int keylen=0;
   for(int i=0; i<numkeys; i++) keylen+=sec[5][i];
   keylen=(keylen+15)&~15;
-  if(sec[6]+keylen>=mem+len) {
+  if(sec[6]+keylen>=d+datahdr->dlen) {
     PRINTF(L_SYS_TPSAU,"section 6 exceeds buffer");
     return false;
     }
@@ -501,7 +775,7 @@ bool cTpsKeys::ProcessAu(unsigned char *mem, int len)
 
   cSimpleList<cTpsKey> *nlist=new cSimpleList<cTpsKey>;
   for(int i=0; i<seclen; i++) {
-    static const unsigned char startkey[] = { 0x01,0x01,0x03 };
+    static const unsigned char startkey[] = { 0x01,0x01 };
     static const unsigned char startaes[] = { 0x09,0x10 };
     static const unsigned char startse[] = { 0x0a,0x10 };
     unsigned char tmp[56];
@@ -512,13 +786,20 @@ bool cTpsKeys::ProcessAu(unsigned char *mem, int len)
     if(CheckFF(tmp,4)) continue;
     int keyid=sec[4][i];
     int keylen=sec[5][keyid];
-    if(keylen<45) continue;
-    unsigned char *tkey=sec[6];
+    if(keylen<32) continue;
+    const unsigned char *tkey=sec[6];
     for(int j=0; j<keyid; j++) tkey+=sec[5][j];
+
+    unsigned char ke[128];
+    if(keylen!=45) {
+      if(!Handle80008003(tkey,keylen,ke)) continue;
+      tkey=ke;
+      }
+
     if(memcmp(tkey,startkey,sizeof(startkey))) continue;
     tmp[52]=0;
-    tmp[53]=tkey[4];
-    tmp[54]=tkey[5];
+    tmp[53]=tkey[5]; //tkey[4];
+    tmp[54]=1;       //tkey[5];
     tmp[55]=0x1c;
     tkey+=9;
     if(memcmp(tkey,startaes,sizeof(startaes))) continue;
@@ -536,6 +817,7 @@ bool cTpsKeys::ProcessAu(unsigned char *mem, int len)
   return res;
 }
 
+/*
 bool cTpsKeys::LoadBin(void)
 {
   static const unsigned char mark[] = { 'T','P','S',0 };
@@ -582,18 +864,59 @@ void cTpsKeys::DecryptBin(const unsigned char *in, unsigned char *out)
     *((unsigned int *)out)=(var1<<(i+2)) | (var1>>(32-(i+2)));
     }
 }
+*/
 
 bool cTpsKeys::ParseLine(const char *line, bool fromCache)
 {
   unsigned char tmp[60];
-  if(GetHex(line,tmp,sizeof(tmp))) {
-    unsigned int crc=crc32_le(0,&tmp[4],sizeof(tmp)-4);
-    if(*((unsigned int *)tmp)==crc) {
-      cTpsKey *k=new cTpsKey;
-      if(k) { k->Set(&tmp[4]); list->Add(k); }
-      return true;
+  if(line[0]=='X') {
+    if(line[1]=='S') {
+      if(algomem) PRINTF(L_SYS_TPS,"multiple start extentions during cache load");
+      if(sscanf(&line[2],"%x %x %x %x",&algolen,&cb1off,&cb2off,&cb3off)==4) {
+        free(algomem);
+        if((algomem=MALLOC(unsigned char,algolen))) {
+          algoread=0;
+          return true;
+          }
+        else PRINTF(L_SYS_TPS,"no memory for algo");
+        }
+      else PRINTF(L_SYS_TPS,"bad format in start extention");
+      }
+    else if(line[1]=='C') {
+      if(algomem) {
+        int off, len;
+        unsigned int crc;
+        if(sscanf(&line[2],"%x %x %n",&off,&crc,&len)==2) {
+          line+=len+2;
+          unsigned char buff[210];
+          if((len=GetHex(line,buff,200,false))) {
+            if(crc==crc32_le(0,buff,len) && off>=0 && off+len<=algolen) {
+              memcpy(&algomem[off],buff,len);
+              algoread+=len;
+              if(algoread==algolen) {
+                RegisterAlgo3(algomem,cb1off,cb2off,cb3off,algolen);
+                free(algomem); algomem=0;
+                }
+              return true;
+              }
+            }
+          }
+        PRINTF(L_SYS_TPS,"bad format in code extention");
+        }
+      else PRINTF(L_SYS_TPS,"unexpected code extention");
+      }
+    else PRINTF(L_SYS_TPS,"unknown extention during cache load");
+    }
+  else {
+    if(GetHex(line,tmp,sizeof(tmp))) {
+      unsigned int crc=crc32_le(0,&tmp[4],sizeof(tmp)-4);
+      if(*((unsigned int *)tmp)==crc) {
+        cTpsKey *k=new cTpsKey;
+        if(k) { k->Set(&tmp[4]); list->Add(k); }
+        return true;
+        }
+      else PRINTF(L_SYS_TPS,"CRC failed during cache load");
       }
-    else PRINTF(L_SYS_TPS,"CRC failed during cache load");
     }
   return false;
 }
@@ -602,14 +925,26 @@ bool cTpsKeys::Save(FILE *f)
 {
   cMutexLock lock(this);
   bool res=true;
+  char str[420];
   for(cTpsKey *k=list->First(); k; k=list->Next(k)) {
     unsigned char tmp[60];
     k->Put(&tmp[4]);
     *((unsigned int *)tmp)=crc32_le(0,&tmp[4],sizeof(tmp)-4);
-    char str[128];
     fprintf(f,"%s\n",HexStr(str,tmp,sizeof(tmp)));
     res=(ferror(f)==0 && res);
     }
+  unsigned char *mem;
+  int len=0, cb1=0, cb2=0, cb3=0;
+  if((mem=DumpAlgo3(len,cb1,cb2,cb3))) {
+    fprintf(f,"XS %04X %04X %04X %04X\n",len,cb1,cb2,cb3);
+    res=(ferror(f)==0 && res);
+    for(int i=0; i<len; i+=200) {
+      int l=min(200,len-i);
+      fprintf(f,"XC %04X %08X %s\n",i,crc32_le(0,&mem[i],l),HexStr(str,&mem[i],l));
+      res=(ferror(f)==0 && res);
+      }
+    free(mem);
+    }
   Modified(!res);
   return res;
 }
@@ -621,23 +956,18 @@ bool cTpsKeys::Save(FILE *f)
 cTpsAuHook::cTpsAuHook(void)
 :cLogHook(HOOK_TPSAU,"tpsau")
 {
-  mem=MALLOC(unsigned char,BUFF_SIZE);
-  pos=0; pmtpid=aupid=-1;
+  mod=0; pmtpid=aupid=-1;
   pids.AddPid(0x0000,0x00,0xff); // PAT
-/*
-  aupid=0x12f2;
-  pids.AddPid(aupid,0x87,0xff);
-*/
 }
 
 cTpsAuHook::~cTpsAuHook()
 {
-  free(mem);
+  delete mod;
 }
 
 void cTpsAuHook::Process(int pid, unsigned char *data)
 {
-  if(data && mem) {
+  if(data) {
     if(pid==0) { // PAT
       SI::PAT pat(data,false);
       if(pat.CheckCRCAndParse()) {
@@ -681,15 +1011,17 @@ void cTpsAuHook::Process(int pid, unsigned char *data)
         }
       }
     else if(pid==aupid) {
-      int len=SCT_LEN(data)-28;
-      if(len>0 && pos+len<BUFF_SIZE) {
-        memcpy(mem+pos,data+24,len);
-        pos+=len;
-        }
-      if(pos>(BUFF_SIZE-BUFF_SIZE/10)) {
-        if(tpskeys.ProcessAu(mem,pos)) BailOut();
-        pos-=BUFF_SIZE/2;
-        memmove(mem,mem+BUFF_SIZE/2,pos);
+      if(cOpenTVModule::Id(data)==2) {
+        if(!mod) mod=new cOpenTVModule(data);
+        if(mod) {
+          int r=mod->AddPart(data,SCT_LEN(data));
+          if(r>0) {
+            PRINTF(L_SYS_TPSAU,"received complete OpenTV module ID 2");
+            if(tpskeys.ProcessAu(mod)) BailOut();
+            r=-1;
+            }            
+          if(r<0) { delete mod; mod=0; }
+          }
         }
       }
     }
@@ -697,17 +1029,113 @@ void cTpsAuHook::Process(int pid, unsigned char *data)
 
 // -- cTPSDecrypt --------------------------------------------------------------
 
+unsigned char *cTPSDecrypt::mem=0;
+int cTPSDecrypt::memLen=0;
+int cTPSDecrypt::cb1off=0;
+int cTPSDecrypt::cb2off=0;
+int cTPSDecrypt::cb3off=0;
+cMutex cTPSDecrypt::st20Mutex;
+cST20 cTPSDecrypt::st20;
+bool cTPSDecrypt::st20Inited=false;
+
 void cTPSDecrypt::TpsDecrypt(unsigned char *data, short mode, const unsigned char *key)
 {
   switch(mode) {
-    case 0:
-    case 3: break;
+    case 0: break;
     case 1: cAES::SetKey(key); cAES::Decrypt(data,16); break;
     case 2: cRC6::SetKey(key,16); cRC6::Decrypt(data); break;
+    case 3: if(mem) {
+              if(!DecryptAlgo3(key,data))
+                PRINTF(L_SYS_TPS,"decrypt failed in algo 3");
+              }
+            else PRINTF(L_SYS_TPS,"no callbacks for algo 3 registered");
+            break;
     default: PRINTF(L_SYS_TPS,"unknown TPS decryption algo %d",mode); break;
     }
 }
 
+bool cTPSDecrypt::RegisterAlgo3(const unsigned char *data, int cb1, int cb2, int cb3, int kd)
+{
+  cMutexLock lock(&st20Mutex);
+  free(mem);
+  if(!(mem=MALLOC(unsigned char,kd))) return false;
+  memcpy(mem,data,kd);
+  memLen=kd; cb1off=cb1; cb2off=cb2; cb3off=cb3;
+  st20Inited=false;
+  PRINTF(L_SYS_TPSAU,"registered callbacks for algo 3");
+  return true;
+}
+
+unsigned char *cTPSDecrypt::DumpAlgo3(int &len, int &cb1, int &cb2, int &cb3)
+{
+  cMutexLock lock(&st20Mutex);
+  if(!mem) return 0;
+  unsigned char *buff=MALLOC(unsigned char,memLen);
+  if(!buff) return 0;
+  memcpy(buff,mem,memLen);
+  len=memLen; cb1=cb1off; cb2=cb2off; cb3=cb3off;
+  return buff;
+}
+
+bool cTPSDecrypt::InitST20(void)
+{
+  if(!mem) return false;
+  if(!st20Inited) {
+    st20.SetFlash(mem,memLen);
+    st20.SetRam(NULL,0x10000);
+    st20Inited=true;
+    }
+  return true;
+}
+
+bool cTPSDecrypt::Handle80008003(const unsigned char *src, int len, unsigned char *dest)
+{
+  cMutexLock lock(&st20Mutex);
+  if(cb1off && InitST20()) {
+    for(int i=0; i<len; i++) st20.WriteByte(RAMS+0x400+i,src[i]);
+    st20.WriteShort(RAMS+0x0,0x8000);
+    st20.WriteShort(RAMS+0x2,0x8003);
+    st20.WriteWord(RAMS+0x8,RAMS+0x400);
+    st20.Init(FLASHS+cb1off,RAMS+0xF000);
+    st20.SetCallFrame(0,RAMS,0,0);
+    int err=st20.Decode(1000);
+    if(err<0) {
+      PRINTF(L_SYS_TPSAU,"ST20 processing failed in callback1 (%d)",err);
+      return false;
+      }
+    for(int i=0; i<0x2D; i++) dest[i]=st20.ReadByte(RAMS+0x400+i);
+    return true;
+    }
+  return false;
+}
+
+bool cTPSDecrypt::DecryptAlgo3(const unsigned char *key, unsigned char *data)
+{
+  cMutexLock lock(&st20Mutex);
+  if(cb2off && cb3off && InitST20()) {
+    for(int i=0; i<16; i++) st20.WriteByte(RAMS+0x400+i,key[i]);
+    st20.Init(FLASHS+cb2off,RAMS+0xF000);
+    st20.SetCallFrame(0,RAMS+0x400,RAMS+0x800,0);
+    int err=st20.Decode(30000);
+    if(err<0) {
+      PRINTF(L_SYS_TPS,"ST20 processing failed in callback2 (%d)",err);
+      return false;
+      }
+
+    for(int i=0; i<16; i++) st20.WriteByte(RAMS+0x400+i,data[i]);
+    st20.Init(FLASHS+cb3off,RAMS+0xF000);
+    st20.SetCallFrame(0,RAMS+0x400,RAMS+0x1000,RAMS+0x800);
+    err=st20.Decode(40000);
+    if(err<0) {
+      PRINTF(L_SYS_TPS,"ST20 processing failed in callback3 (%d)",err);
+      return false;
+      }
+    for(int i=0; i<16; i++) data[i]=st20.ReadByte(RAMS+0x1000+i);
+    return true;
+    }
+  return false;
+}
+
 // -- cTPS ---------------------------------------------------------------------
 
 cTPS::cTPS(void)
@@ -734,19 +1162,20 @@ int cTPS::Decrypt(int cardNum, int Source, int Transponder, unsigned char *data,
   if(now<0) return -1;
 
   const cTpsKey *k=tpskeys.GetKey(now);
-  if(!k) {
-    PRINTF(L_SYS_TPS,"no TPS key for current time. Outdated "TPSBIN" file?");
-    return -1;
-    }
   doPost=0;
-  int opmode=k->Opmode(), doTPS=0, doPre=0, hasDF=0, ret=0;
+  int opmode=k?k->Opmode():4, doTPS=0, doPre=0, hasDF=0, ret=0;
   if((opmode&4) && data[0]==0xD2 && data[1]==0x01 && data[2]==0x01) {
     data+=3; len-=3; ret=3;
     if(data[0]==0x90) PRINTF(L_SYS_TPS,"TPS v1 is no longer supported");
     if(data[0]==0x40) data[0]=0x90;
     doTPS=1;
+    if(!k) k=tpskeys.GetV2Key(0x7c00);
     }
-  for(int i=0; i<len; i+=data[i+1]+2)
+  for(int i=0; i<len; i+=data[i+1]+2) {
+    if(!k && (doTPS || doPre || doPost)) {
+      PRINTF(L_SYS_TPS,"no TPS key available for current time of day");
+      return -1;
+      }
     switch(data[i]) {
       case 0xDF:
         if(!(opmode&4)) doTPS =(0x6996>>((data[i+2]&0xF)^(data[i+2]>>4)))&1;
@@ -756,10 +1185,11 @@ int cTPS::Decrypt(int cardNum, int Source, int Transponder, unsigned char *data,
         break;
       case 0xEA:
         if(doPre) TpsDecrypt(&data[i+2],k->Mode(0),k->Key(0));
-        if(doTPS) TpsDecrypt(&data[i+2],(hasDF)?k->Mode(1):1,k->Key(1));
+        if(doTPS) TpsDecrypt(&data[i+2],(hasDF)?k->Mode(2):1,k->Key(2));
+        if(doPost) { postMode=k->Mode(1); memcpy(postKey,k->Key(1),sizeof(postKey)); }
         break;
       }
-  postMode=k->Mode(2); memcpy(postKey,k->Key(2),sizeof(postKey));
+    }
   return ret;
 }
 
