@@ -40,8 +40,15 @@
 #error "This version of ttxtsubs only works with vdr version >= 1.4.0!"
 #endif
 
+#ifndef trNOOP
+#define trNOOP(s) (s)
+#endif
+#ifndef trVDR
+#define trVDR(s) tr(s)
+#endif
+
 static const char *VERSION        = "0.0.5-RRE";
-static const char *DESCRIPTION    = "Teletext subtitles";
+static const char *DESCRIPTION    = trNOOP("Teletext subtitles");
 
 cTtxtsubsConf globals;
 cTtxtSubsChannelSettings TtxtSubsChannelSettings;
@@ -71,7 +78,9 @@ char *gLanguages[][2] = {
   {"rus",""},    //Russian
   {"hrv","scr"}, //Croatian
   {"est",""},    //Estonian
-  {"dan",""}     //Danish
+  {"dan",""},    //Danish
+  {"cze","ces"}, //Czech
+  {"tur",""}     //Turkish
 };
 const char *gLanguageNames[] = {
   "-",
@@ -86,23 +95,17 @@ const char *gLanguageNames[] = {
   "suomi", // this is not a typo - it's really lowercase!
   "Polski",
   "Espaol",
-#if VDRVERSNUM >= 10302
   "",
-#else
-  "Ellinika",
-#endif
   "Svenska",
   "Romaneste",
   "Magyar",
   "Catal",
-#if VDRVERSNUM >= 10302
   "",
-#else
-  "Russian",
-#endif
   "Hrvatski",
   "Eesti",
-  "Dansk"
+  "Dansk",
+  "esky",
+  "Trke"
 };
 int gNumLanguages = sizeof(gLanguages) / sizeof(gLanguages[0]);
 
@@ -210,9 +213,6 @@ cPluginTtxtsubs::cPluginTtxtsubs(void)
 cPluginTtxtsubs::~cPluginTtxtsubs()
 {
   // Clean up after yourself!
-#if VDRVERSNUM < 10320
-  Stop();
-#endif
 }
 
 const char *cPluginTtxtsubs::CommandLineHelp(void)
@@ -234,9 +234,9 @@ bool cPluginTtxtsubs::Start(void)
   cThread::Start();
 
   TtxtSubsChannelSettings.Load(AddDirectory(ConfigDirectory("ttxtsubs"),"channelsettings.dat"));
-
+#if defined(APIVERSNUM) && APIVERSNUM < 10507
   RegisterI18n(Phrases);
-
+#endif
   if(!memcmp(globals.mLanguages[0][0], "unk", 3)) {
     // no language found in setup
     if(strlen(mOldLanguage)) {
@@ -259,6 +259,7 @@ bool cPluginTtxtsubs::Start(void)
       globals.mHearingImpaireds[0][1] = mOldHearingImpaired;
     } else {
       // get lang from OSD lang
+#if defined(APIVERSNUM) && APIVERSNUM < 10507
       int n = Setup.OSDLanguage + 1;
       if(n > gNumLanguages) {
 	strncpy(globals.mLanguages[0][0], "eng", 4);
@@ -268,6 +269,10 @@ bool cPluginTtxtsubs::Start(void)
 	strncpy(globals.mLanguages[0][1], gLanguages[n][1], 4);
 	globals.mLanguages[0][1][3] = '\0';
       }
+#else
+      strncpy(globals.mLanguages[0][0], Setup.OSDLanguage, 4);
+      globals.mLanguages[0][0][3] = '\0';
+#endif
     }
   }
 
@@ -304,8 +309,6 @@ const char *cPluginTtxtsubs::MainMenuEntry(void)
       return tr("Position Teletext Subtitles for Letterbox");
   case 3:
     return tr("Page Selection");
-  case 4:
-    return tr("Teletext Subtitles");
   default:
     return NULL;
   }
@@ -326,8 +329,6 @@ cOsdObject *cPluginTtxtsubs::MainMenuAction(void)
     return NULL;
   case 3:
     return new cTtxtsubsPageMenu();
-  case 4:
-    return new cMenuSetupTtxtsubs(this, 1);
   default:
     return NULL;
   }
@@ -353,6 +354,9 @@ bool cPluginTtxtsubs::SetupParse(const char *Name, const char *Value)
   else if(!strcasecmp(Name, "FrenchSpecial")) globals.mFrenchSpecial = atoi(Value);
   else if(!strcasecmp(Name, "LineSpacing")) globals.mLineSpacing = atoi(Value);
   else if(!strcasecmp(Name, "DvbSources")) globals.mDvbSources = atoi(Value);
+#if defined(APIVERSNUM) && APIVERSNUM >= 10503
+  else if(!strcasecmp(Name, "FontSize")) globals.mFontSize = atoi(Value);
+#endif
   else if(!strcasecmp(Name, "FgColor")) globals.mFgColor = atoi(Value);
   else if(!strcasecmp(Name, "BgColor")) globals.mBgColor = atoi(Value);
   else if(!strcasecmp(Name, "CustomColor")) globals.mCustomColor = atoi(Value);
@@ -419,7 +423,11 @@ void cPluginTtxtsubs::Action(void)
                         globals.mI18nLanguage = i - 1;
                       }
                    }
+#if defined(APIVERSNUM) && APIVERSNUM < 10507
                    if(globals.mI18nLanguage < 0 || globals.mI18nLanguage >= I18nNumLanguages)
+#else
+                   if(globals.mI18nLanguage < 0 || globals.mI18nLanguage >= I18nLanguages()->Size())
+#endif
                      globals.mI18nLanguage = 0; // default to iso8859-1 if no predefined charset
                    StartTtxtLive(dev, c->GetChannelID(), pid, page);
                    FreeTtxtInfoData(&info);
@@ -625,12 +633,11 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   mDoStore(doStore),
   mConf(globals)
 {
-  int tmp;
   char *buf = NULL;
   //static char *mainMenuAlts[] = {"off", "Display on/off", "4:3/Letterbox", "This menu"};
   // can't get it to store changes in file
   if(mainMenuAlts[0] == NULL) {
-    mainMenuAlts[0] = tr("off");
+    mainMenuAlts[0] = trVDR("off");
     mainMenuAlts[1] = tr("Display on/off");
     mainMenuAlts[2] = tr("4:3/Letterbox");
     mainMenuAlts[3] = tr("Page Mode");
@@ -657,7 +664,7 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
     textColors[7] = tr("Cyan");
     textColors[8] = tr("Custom");
     textColors[9] = tr("Transparent");
-    textColors[10] = tr("Background");
+    textColors[10] = tr("Grey");
     textColors[11] = NULL;
   }
   const int numTextPosAlts = sizeof(textPosAlts) / sizeof(textPosAlts[0]) - 1;
@@ -681,8 +688,8 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
     }
   }
 
-  Add(new cMenuEditBoolItem(tr("Display Subtitles"), &mConf.mDoDisplay, tr("no"), tr("yes")));
-  Add(new cMenuEditBoolItem(tr("Record Subtitles"), &mConf.mDoRecord, tr("no"), tr("yes")));
+  Add(new cMenuEditBoolItem(tr("Display Subtitles"), &mConf.mDoDisplay));
+  Add(new cMenuEditBoolItem(tr("Record Subtitles"), &mConf.mDoRecord));
   Add(new cMenuEditIntItem(tr("Live Delay"), &mConf.mLiveDelay, 0, 5000));
   Add(new cMenuEditIntItem(tr("Replay Delay"), &mConf.mReplayDelay, 0, 5000));
   if(mConf.mMainMenuEntry < 0 || mConf.mMainMenuEntry >= numMainMenuAlts)
@@ -700,11 +707,14 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   Add(new cMenuEditIntItem( tr("Line Spacing Adjust"),
 			    &mConf.mLineSpacing, -25, 25));
   Add(new cMenuEditBoolItem(tr("Workaround for some French chns"),
-			    &mConf.mFrenchSpecial, tr("no"), tr("yes")));
+			    &mConf.mFrenchSpecial));
   if(mConf.mDvbSources < 0 || mConf.mDvbSources >= numDvbSources)
     mConf.mDvbSources = 0;  // menu item segfaults if out of range
   Add(new cMenuEditStraItem(tr("DVB Source Selection"),
                           &mConf.mDvbSources, 4, dvbSources));
+#if defined(APIVERSNUM) && APIVERSNUM >= 10503
+  Add(new cMenuEditIntItem(tr("Font Size (pixel)"), &mConf.mFontSize, 10, MAXFONTSIZE));
+#endif
   if(mConf.mFgColor < 0 || mConf.mFgColor >= numTextColors)
     mConf.mFgColor = 1;  // menu item segfaults if out of range
   Add(new cMenuEditStraItem(tr("Text Color"), &mConf.mFgColor,
@@ -723,7 +733,7 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   if (mRed < 0 || mRed >= numColorValues) mRed = 0;
   mTransparency = (mConf.mCustomColor & 0x0F000000) >> 24;
   if (mTransparency < 0 || mTransparency >= numColorValues) mTransparency = 0;
-  tmp = asprintf(&buf, "%s: --------------------------------------------", tr("Custom Color"));
+  asprintf(&buf, "%s: --------------------------------------------", tr("Custom Color"));
   cOsdItem *item = new cOsdItem(buf);
   item->SetSelectable(false);
   free(buf);
@@ -750,8 +760,7 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
     }
 
     sprintf(str, "%s %d %s", tr("Language"), n + 1, tr("Hearing Impaired"));
-    Add(new cMenuEditBoolItem(str, &(mConf.mHearingImpaireds[n][0]),
-			      tr("no"), tr("yes")));
+    Add(new cMenuEditBoolItem(str, &(mConf.mHearingImpaireds[n][0])));
   }
 
 #ifdef TEST
@@ -792,6 +801,9 @@ void cMenuSetupTtxtsubs::Store(void)
   SetupStore("MainMenuEntry", mConf.mMainMenuEntry);
   SetupStore("LineSpacing", mConf.mLineSpacing);
   SetupStore("DvbSources", mConf.mDvbSources);
+#if defined(APIVERSNUM) && APIVERSNUM >= 10503
+  SetupStore("FontSize", mConf.mFontSize);
+#endif
   SetupStore("FgColor", mConf.mFgColor);
   SetupStore("BgColor", mConf.mBgColor);
   mConf.mCustomColor = (mBlue              ) | (mBlue         <<  4) |
