@@ -30,10 +30,12 @@ int MVPClient::nr_clients = 0;
 MVPClient::MVPClient(Config* cfgBase, char* tconfigDirExtra, int tsocket)
  : tcp(tsocket)
 {
+#ifndef VOMPSTANDALONE
   lp = NULL;
   rp = NULL;
-  imageFile = 0;
   recordingManager = NULL;
+#endif
+  imageFile = 0;
   log = Log::getInstance();
   loggedIn = false;
   configDirExtra = tconfigDirExtra;
@@ -44,6 +46,7 @@ MVPClient::MVPClient(Config* cfgBase, char* tconfigDirExtra, int tsocket)
 MVPClient::~MVPClient()
 {
   log->log("Client", Log::DEBUG, "MVP client destructor");
+#ifndef VOMPSTANDALONE
   if (lp)
   {
     delete lp;
@@ -58,7 +61,7 @@ MVPClient::~MVPClient()
     rp = NULL;
     recordingManager = NULL;
   }
-
+#endif
   if (loggedIn) cleanConfig();
   decClients();
 }
@@ -88,6 +91,7 @@ ULLONG MVPClient::htonll(ULLONG a)
   #endif
 }
 
+#ifndef VOMPSTANDALONE
 cChannel* MVPClient::channelFromNumber(ULONG channelNumber)
 {
   cChannel* channel = NULL;
@@ -126,6 +130,7 @@ void MVPClient::writeResumeData()
                           (char*)rp->getCurrentRecording()->FileName(),
                           rp->frameNumberFromPosition(rp->getLastPosition()) );
 }
+#endif
 
 void MVPClient::sendULONG(ULONG ul)
 {
@@ -167,111 +172,162 @@ void MVPClient::run2()
   tcp.setSoKeepTime(3);
   tcp.setNonBlocking();
 
-  UCHAR* buffer;
-  UCHAR* data;
-  int packetLength;
+  ULONG channelID;
+  ULONG serialNumber;
   ULONG opcode;
+  ULONG extraDataLength;
+  UCHAR* data;
+    
   int result = 0;
 
   while(1)
   {
     log->log("Client", Log::DEBUG, "Waiting");
-    buffer = (UCHAR*)tcp.receivePacket();
-    log->log("Client", Log::DEBUG, "Received packet, length = %u", tcp.getDataLength());
-    if (buffer == NULL)
+    
+    if (!tcp.readData((UCHAR*)&channelID, sizeof(ULONG))) break;
+    channelID = ntohl(channelID);
+    if (channelID != 1)
     {
-      log->log("Client", Log::DEBUG, "Detected connection closed");
+      log->log("Client", Log::ERR, "Incoming channel number not 1!");
       break;
     }
 
-    packetLength = tcp.getDataLength() - 4;
-    opcode = ntohl(*(ULONG*)buffer);
-    data = buffer + 4;
+    log->log("Client", Log::DEBUG, "Got chan");
+    
+    if (!tcp.readData((UCHAR*)&serialNumber, sizeof(ULONG))) break;
+    serialNumber = ntohl(serialNumber);
+
+    log->log("Client", Log::DEBUG, "Got ser");
+
+    if (!tcp.readData((UCHAR*)&opcode, sizeof(ULONG))) break;
+    opcode = ntohl(opcode);
+
+    log->log("Client", Log::DEBUG, "Got op %lu", opcode);
+
+    if (!tcp.readData((UCHAR*)&extraDataLength, sizeof(ULONG))) break;
+    extraDataLength = ntohl(extraDataLength);
+    if (extraDataLength > 200000)
+    {
+      log->log("Client", Log::ERR, "ExtraDataLength > 200000!");
+      break;
+    }
+
+    log->log("Client", Log::DEBUG, "Got edl %lu", extraDataLength);
+
+    if (extraDataLength)
+    {
+      data = (UCHAR*)malloc(extraDataLength);
+      if (!data)
+      {
+        log->log("Client", Log::ERR, "Extra data buffer malloc error");
+        break;
+      }
+      
+      if (!tcp.readData(data, extraDataLength))
+      {
+        log->log("Client", Log::ERR, "Could not read extradata");
+        free(data);
+        break;
+      }      
+    }
+    else
+    {
+      data = NULL;
+    }
+
+    log->log("Client", Log::DEBUG, "Received chan=%lu, ser=%lu, op=%lu, edl=%lu", channelID, serialNumber, opcode, extraDataLength);
 
     if (!loggedIn && (opcode != 1))
     {
-      free(buffer);
+      log->log("Client", Log::ERR, "Not logged in and opcode != 1");
+      if (data) free(data);
       break;
     }
 
-    log->log("Client", Log::DEBUG, "SwitchOp");
     switch(opcode)
     {
       case 1:
-        result = processLogin(data, packetLength);
+        result = processLogin(data, extraDataLength);
         break;
+#ifndef VOMPSTANDALONE
       case 2:
-        result = processGetRecordingsList(data, packetLength);
+        result = processGetRecordingsList(data, extraDataLength);
         break;
       case 3:
-        result = processDeleteRecording(data, packetLength);
+        result = processDeleteRecording(data, extraDataLength);
         break;
       case 5:
-        result = processGetChannelsList(data, packetLength);
+        result = processGetChannelsList(data, extraDataLength);
         break;
       case 6:
-        result = processStartStreamingChannel(data, packetLength);
+        result = processStartStreamingChannel(data, extraDataLength);
         break;
       case 7:
-        result = processGetBlock(data, packetLength);
+        result = processGetBlock(data, extraDataLength);
         break;
       case 8:
-        result = processStopStreaming(data, packetLength);
+        result = processStopStreaming(data, extraDataLength);
         break;
       case 9:
-        result = processStartStreamingRecording(data, packetLength);
+        result = processStartStreamingRecording(data, extraDataLength);
         break;
       case 10:
-        result = processGetChannelSchedule(data, packetLength);
+        result = processGetChannelSchedule(data, extraDataLength);
         break;
+#endif
       case 11:
-        result = processConfigSave(data, packetLength);
+        result = processConfigSave(data, extraDataLength);
         break;
       case 12:
-        result = processConfigLoad(data, packetLength);
+        result = processConfigLoad(data, extraDataLength);
         break;
+#ifndef VOMPSTANDALONE
       case 13:
-        result = processReScanRecording(data, packetLength);         // FIXME obselete
+        result = processReScanRecording(data, extraDataLength);         // FIXME obselete
         break;
       case 14:
-        result = processGetTimers(data, packetLength);
+        result = processGetTimers(data, extraDataLength);
         break;
       case 15:
-        result = processSetTimer(data, packetLength);
+        result = processSetTimer(data, extraDataLength);
         break;
       case 16:
-        result = processPositionFromFrameNumber(data, packetLength);
+        result = processPositionFromFrameNumber(data, extraDataLength);
         break;
       case 17:
-        result = processFrameNumberFromPosition(data, packetLength);
+        result = processFrameNumberFromPosition(data, extraDataLength);
         break;
       case 18:
-        result = processMoveRecording(data, packetLength);
+        result = processMoveRecording(data, extraDataLength);
         break;
       case 19:
-        result = processGetIFrame(data, packetLength);
+        result = processGetIFrame(data, extraDataLength);
         break;
       case 20:
-        result = processGetRecInfo(data, packetLength);
+        result = processGetRecInfo(data, extraDataLength);
         break;
       case 21:
-        result = processGetMarks(data, packetLength);
+        result = processGetMarks(data, extraDataLength);
         break;
       case 22:
-        result = processGetChannelPids(data, packetLength);
+        result = processGetChannelPids(data, extraDataLength);
         break;
+      case 23:
+        result = processDeleteTimer(data, extraDataLength);
+        break;
+#endif
       case 30:
-        result = processGetMediaList(data, packetLength);
+        result = processGetMediaList(data, extraDataLength);
         break;
       case 31:
-        result = processGetPicture(data, packetLength);
+        result = processGetPicture(data, extraDataLength);
         break;
       case 32:
-        result = processGetImageBlock(data, packetLength);
+        result = processGetImageBlock(data, extraDataLength);
         break;
     }
 
-    free(buffer);
+    if (data) free(data);
     if (!result) break;
   }
 }
@@ -282,7 +338,11 @@ int MVPClient::processLogin(UCHAR* buffer, int length)
 
   // Open the config
 
+#ifndef VOMPSTANDALONE
   const char* configDir = cPlugin::ConfigDirectory(configDirExtra);
+#else
+  const char* configDir = ".";
+#endif
   if (!configDir)
   {
     log->log("Client", Log::DEBUG, "No config dir!");
@@ -311,6 +371,7 @@ int MVPClient::processLogin(UCHAR* buffer, int length)
   return 1;
 }
 
+#ifndef VOMPSTANDALONE
 int MVPClient::processGetRecordingsList(UCHAR* data, int length)
 {
   UCHAR* sendBuffer = new UCHAR[50000]; // hope this is enough
@@ -692,7 +753,7 @@ int MVPClient::processGetChannelPids(UCHAR* data, int length)
 #endif
 
 //  printf("About to send getchannelpids response. length = %u\n", spaceRequired);
-  tcp.dump(sendBuffer, spaceRequired);
+  //tcp.dump(sendBuffer, spaceRequired);
 
   tcp.sendPacket(sendBuffer, spaceRequired);
   delete[] sendBuffer;
@@ -1003,7 +1064,7 @@ int MVPClient::processGetChannelSchedule(UCHAR* data, int length)
   ULONG sendBufferLength = 100000;
   ULONG sendBufferUsed = sizeof(ULONG); // leave a hole for the entire packet length
 
-  char* empty = "";
+  const char* empty = "";
 
   // assign all the event info to temp vars then we know exactly what size they are
   ULONG thisEventID;
@@ -1115,6 +1176,8 @@ int MVPClient::processGetChannelSchedule(UCHAR* data, int length)
   return 1;
 }
 
+#endif //VOMPSTANDALONE
+
 int MVPClient::processConfigSave(UCHAR* buffer, int length)
 {
   char* section = (char*)buffer;
@@ -1196,6 +1259,7 @@ void MVPClient::cleanConfig()
 {
   log->log("Client", Log::DEBUG, "Clean config");
 
+#ifndef VOMPSTANDALONE
   cRecordings Recordings;
   Recordings.Load();
 
@@ -1223,6 +1287,7 @@ void MVPClient::cleanConfig()
   }
 
   delete[] resumes;
+#endif
 }
 
 
@@ -1536,6 +1601,7 @@ recording is a bool, 0 for not currently recording, 1 for currently recording
 pending is a bool, 0 for would not be trying to record this right now, 1 for would/is trying to record this right now
 */
 
+#ifndef VOMPSTANDALONE
 
 int MVPClient::processGetTimers(UCHAR* buffer, int length)
 {
@@ -1566,6 +1632,8 @@ int MVPClient::processGetTimers(UCHAR* buffer, int length)
     *(ULONG*)&sendBuffer[count] = htonl(timer->Channel()->Number());      count += 4;
     *(ULONG*)&sendBuffer[count] = htonl(timer->StartTime());              count += 4;
     *(ULONG*)&sendBuffer[count] = htonl(timer->StopTime());               count += 4;
+    *(ULONG*)&sendBuffer[count] = htonl(timer->Day());                    count += 4;
+    *(ULONG*)&sendBuffer[count] = htonl(timer->WeekDays());               count += 4;
 
     fileName = timer->File();
     strcpy((char*)&sendBuffer[count], fileName);
@@ -1576,6 +1644,8 @@ int MVPClient::processGetTimers(UCHAR* buffer, int length)
 
   log->log("Client", Log::DEBUG, "recorded size as %u", ntohl(*(ULONG*)&sendBuffer[0]));
 
+//tcp.dump(sendBuffer, count);
+
   tcp.sendPacket(sendBuffer, count);
   delete[] sendBuffer;
   log->log("Client", Log::DEBUG, "Written timers list");
@@ -1648,6 +1718,8 @@ int MVPClient::processSetTimer(UCHAR* buffer, int length)
   return 1;
 }
 
+#endif //VOMPSTANDALONE
+
 void MVPClient::incClients()
 {
   pthread_mutex_lock(&threadClientMutex);
@@ -1671,6 +1743,8 @@ int MVPClient::getNrClients()
   return nrClients;
 }
 
+#ifndef VOMPSTANDALONE
+
 int MVPClient::processGetRecInfo(UCHAR* data, int length)
 {
   // data is a pointer to the fileName string
@@ -1941,6 +2015,8 @@ int MVPClient::processGetMarks(UCHAR* data, int length)
   return 1;
 }
 
+#endif //VOMPSTANDALONE
+
 
 /**
   * media List Request:
@@ -1988,10 +2064,10 @@ int MVPClient::processGetMediaList(UCHAR* data, int length)
   *(ULONG*)&sendBuffer[count] = htonl(0);
   count += 4;
   //numentries
-  *(ULONG*)&sendBuffer[count] = htonl(ml->Count());
+  *(ULONG*)&sendBuffer[count] = htonl(ml->size());
   count += 4;
-  for (int nm=0;nm<ml->Count() && count < (MLISTBUF-1000);nm++) {
-      Media *m=ml->Get(nm);
+  for (MediaList::iterator nm=ml->begin();nm<ml->end() && count < (MLISTBUF-1000);nm++) {
+      Media *m=*nm;
       log->log("Client", Log::DEBUG, "found media entry %s, type=%d",m->getFilename(),m->getType());
       *(ULONG*)&sendBuffer[count] = htonl(m->getType());
       count += 4;
@@ -2024,8 +2100,6 @@ int MVPClient::processGetMediaList(UCHAR* data, int length)
 
 /**
   * get image Request:
-  * 4 length
-  * 4 VDR_GETIMAGE
   * 4 flags (currently unused)
   * 4 x size
   * 4 y size
@@ -2048,9 +2122,9 @@ int MVPClient::processGetPicture(UCHAR* data, int length)
     imageFile=NULL;
   }
   char * filename=NULL;
-  if (length > 4) {
+  if (length > 12) {
     //we have a dirname provided
-    filename=(char *)&data[4];
+    filename=(char *)&data[12];
     log->log("Client", Log::DEBUG, "getPicture  %s", filename);
   }
   else {
@@ -2126,4 +2200,59 @@ int MVPClient::processGetImageBlock(UCHAR* data, int length)
   return 1;
 }
 
+#ifndef VOMPSTANDALONE
 
+int MVPClient::processDeleteTimer(UCHAR* buffer, int length)
+{
+  log->log("Client", Log::DEBUG, "Delete timer called");
+  // get timer
+  
+  int position = 0;
+  
+  INT delChannel = ntohl(*(ULONG*)&buffer[position]); position += 4;
+  INT delWeekdays = ntohl(*(ULONG*)&buffer[position]); position += 4;
+  INT delDay = ntohl(*(ULONG*)&buffer[position]); position += 4;  
+  INT delStart = ntohl(*(ULONG*)&buffer[position]); position += 4;  
+  INT delStop = ntohl(*(ULONG*)&buffer[position]); position += 4;
+    
+  cTimer* ti = NULL;
+  for (ti = Timers.First(); ti; ti = Timers.Next(ti))
+  {
+    if  ( (ti->Channel()->Number() == delChannel)
+     &&   ((ti->WeekDays() && (ti->WeekDays() == delWeekdays)) || (!ti->WeekDays() && (ti->Day() == delDay)))
+     &&   (ti->StartTime() == delStart)
+     &&   (ti->StopTime() == delStop) )
+       break;
+  }
+  
+  if (!ti)
+  {
+    sendULONG(4);
+    return 1;
+  }
+          
+  if (!Timers.BeingEdited())
+  {
+    if (!ti->Recording())
+    {
+      Timers.Del(ti);
+      Timers.SetModified();
+      sendULONG(10);
+      return 1;
+    }
+    else
+    {
+      log->log("Client", Log::ERR, "Unable to delete timer - timer is running");
+      sendULONG(3);
+      return 1;
+    }  
+  }
+  else
+  {
+    log->log("Client", Log::ERR, "Unable to delete timer - timers being edited at VDR");
+    sendULONG(1);
+    return 1;
+  }  
+}
+
+#endif
