@@ -8,6 +8,7 @@
 
 #include "yaepg.h"
 #include <vdr/device.h>
+#include <ctype.h> 
 
 #define EPGWinHandle		areas[0]
 #define InfoWinHandle		areas[1]
@@ -718,39 +719,36 @@ cInfoBox::~cInfoBox()
 void
 cInfoBox::FormatText(void)
 {
-	char *desc, *token, *sp, line[INFBOX_TXT_WIDTH];
+  const char *noInfo = tr("No Info"); 
+  char line[INFBOX_TXT_WIDTH]; 
+  const char *startPtr = noInfo; 
+  int breakPos, endPos; 
 
-	if (event->Description() != NULL) {
-		desc = strdup(event->Description());
-		if (desc == NULL) {
-			fprintf(stderr, "strdup() returned NULL!\n");
-			return;
-		}
-	}
-	else {
-		desc = strdup(tr("No Info"));
-	}
+  if (event->Description() != NULL) { 
+     startPtr = event->Description(); 
+     }
+	
+  #ifndef MIN
+  # define MIN(a,b) ((a<b)?a:b) 
+  #endif 
 
-	bzero(line, INFBOX_TXT_WIDTH);
-	for (token = strtok_r(desc, " ", &sp);
-			token;
-			token = strtok_r(NULL, " ", &sp)) {
-		if (strlen(line) + strlen(token) + 1 >= INFBOX_TXT_WIDTH) {
-			AddLine(line);
-			bzero(line, INFBOX_TXT_WIDTH);
-		}
-		strncat(line, " ", INFBOX_TXT_WIDTH);
-		strncat(line, token, INFBOX_TXT_WIDTH);
-	}
+  while (*startPtr) { 
+     endPos = MIN((INFBOX_TXT_WIDTH - 1),strlen(startPtr));
+     for (breakPos = endPos; 
+        !isspace(startPtr[breakPos]) && breakPos; 
+	breakPos--); 
+     // Space found 
+     if (breakPos) 
+        endPos = breakPos; 
 
-	/*
-	 * If there is data in line, make sure it gets added.
-	 */
-	if (strlen(line) != 0) {
-		AddLine(line);
-	}
+     bzero(line, INFBOX_TXT_WIDTH); 
+     strncpy(line, startPtr, endPos); 
+     AddLine(line); 
+     startPtr = startPtr + endPos; 
 
-	free(desc);
+     // Skip the space if one found 
+     if (breakPos) startPtr++; 
+     }
 }
 
 #define DTBOX_WIDTH			314
