@@ -166,6 +166,7 @@ cChannel::cChannel(void)
   shortName = strdup("");
   provider = strdup("");
   portalName = strdup("");
+  pluginParam = strdup("");
   memset(&__BeginData__, 0, (char *)&__EndData__ - (char *)&__BeginData__);
   inversion    = INVERSION_AUTO;
   bandwidth    = BANDWIDTH_AUTO;
@@ -187,6 +188,7 @@ cChannel::cChannel(const cChannel &Channel)
   shortName = NULL;
   provider = NULL;
   portalName = NULL;
+  pluginParam = NULL;
   schedule     = NULL;
   linkChannels = NULL;
   refChannel   = NULL;
@@ -215,6 +217,7 @@ cChannel::~cChannel()
   free(shortName);
   free(provider);
   free(portalName);
+  free(pluginParam);
 }
 
 cChannel& cChannel::operator= (const cChannel &Channel)
@@ -223,6 +226,7 @@ cChannel& cChannel::operator= (const cChannel &Channel)
   shortName = strcpyrealloc(shortName, Channel.shortName);
   provider = strcpyrealloc(provider, Channel.provider);
   portalName = strcpyrealloc(portalName, Channel.portalName);
+  pluginParam = strcpyrealloc(pluginParam, Channel.pluginParam);
   memcpy(&__BeginData__, &Channel.__BeginData__, (char *)&Channel.__EndData__ - (char *)&Channel.__BeginData__);
   return *this;
 }
@@ -280,9 +284,26 @@ void cChannel::CopyTransponderData(const cChannel *Channel)
      transmission = Channel->transmission;
      guard        = Channel->guard;
      hierarchy    = Channel->hierarchy;
+     if (IsPlug())  pluginParam = strcpyrealloc(pluginParam, Channel->pluginParam);
      }
 }
 
+bool cChannel::SetPlugTransponderData(int Source, int Frequency, const char *PluginParam)
+{
+  if (source != Source || frequency != Frequency || (strcmp(pluginParam, PluginParam) != 0)) {
+       if (Number()) {
+          dsyslog("changing transponder data of channel %d from %s:%d:%s to %s:%d:%s", Number(), *cSource::ToString(source), frequency, pluginParam, *cSource::ToString(Source), Frequency, PluginParam);
+          modification |= CHANNELMOD_TRANSP;
+          Channels.SetModified();
+          }
+       source = Source;
+       frequency = Frequency;
+       pluginParam = strcpyrealloc(pluginParam, PluginParam);
+       schedule = NULL;
+       }
+    return true;
+}
+
 bool cChannel::SetSatTransponderData(int Source, int Frequency, char Polarization, int Srate, int CoderateH)
 {
   // Workarounds for broadcaster stupidity:
@@ -407,6 +428,18 @@ void cChannel::SetPortalName(const char *PortalName)
      }
 }
 
+void cChannel::SetPluginParam(const char *PluginParam)
+{
+  if (!isempty(PluginParam) && strcmp(pluginParam, PluginParam) != 0) {
+     if (Number()) {
+        dsyslog("changing plugin parameters of channel %d from '%s' to '%s'", Number(), pluginParam, PluginParam);
+        modification |= CHANNELMOD_TRANSP;
+        Channels.SetModified();
+        }
+     pluginParam = strcpyrealloc(pluginParam, PluginParam);
+     }
+}
+
 #define STRDIFF 0x01
 #define VALDIFF 0x02
 
@@ -593,7 +626,7 @@ cString cChannel::ParametersToString(void) const
   if (isdigit(type))
      type = 'S';
 #define ST(s) if (strchr(s, type))
-  char buffer[64];
+  char buffer[256];
   char *q = buffer;
   *q = 0;
   ST(" S ")  q += sprintf(q, "%c", polarization);
@@ -605,6 +638,7 @@ cString cChannel::ParametersToString(void) const
   ST("  T")  q += PrintParameter(q, 'T', MapToUser(transmission, TransmissionValues));
   ST("  T")  q += PrintParameter(q, 'G', MapToUser(guard, GuardValues));
   ST("  T")  q += PrintParameter(q, 'Y', MapToUser(hierarchy, HierarchyValues));
+  ST("P  ")  snprintf(buffer, sizeof(buffer), "%s", pluginParam);
   return buffer;
 }
 
@@ -626,7 +660,7 @@ static const char *ParseParameter(const char *s, int &Value, const tChannelParam
 
 bool cChannel::StringToParameters(const char *s)
 {
-  while (s && *s) {
+  while (s && *s && !IsPlug()) {
         switch (toupper(*s)) {
           case 'B': s = ParseParameter(s, bandwidth, BandwidthValues); break;
           case 'C': s = ParseParameter(s, coderateH, CoderateValues); break;
@@ -736,7 +770,7 @@ bool cChannel::Parse(const char *s)
         dpids[0] = 0;
         ok = false;
         if (parambuf && sourcebuf && vpidbuf && apidbuf) {
-           ok = StringToParameters(parambuf) && (source = cSource::FromString(sourcebuf)) >= 0;
+	   ok = ((source = cSource::FromString(sourcebuf)) >= 0) && StringToParameters(parambuf);
 
            char *p = strchr(vpidbuf, '+');
            if (p)
@@ -827,6 +861,7 @@ bool cChannel::Parse(const char *s)
            shortName = strcpyrealloc(shortName, p);
            }
         name = strcpyrealloc(name, namebuf);
+	if (IsPlug()) pluginParam = strcpyrealloc(pluginParam, parambuf);
 
         free(parambuf);
         free(sourcebuf);
