@@ -267,6 +267,7 @@ cSetup::cSetup(void)
   UpdateChannels = 5;
   RecordDolbyDigital = 1;
   UseDolbyDigital = 1;
+  UseSyncEarlyPatch = 1;
   ChannelInfoPos = 0;
   ChannelInfoTime = 5;
   OSDLeft = 54;
@@ -454,6 +455,7 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "UpdateChannels"))      UpdateChannels     = atoi(Value);
   else if (!strcasecmp(Name, "RecordDolbyDigital"))  RecordDolbyDigital = atoi(Value);
   else if (!strcasecmp(Name, "UseDolbyDigital"))     UseDolbyDigital    = atoi(Value);
+  else if (!strcasecmp(Name, "UseSyncEarlyPatch"))   UseSyncEarlyPatch  = atoi(Value);
   else if (!strcasecmp(Name, "ChannelInfoPos"))      ChannelInfoPos     = atoi(Value);
   else if (!strcasecmp(Name, "ChannelInfoTime"))     ChannelInfoTime    = atoi(Value);
   else if (!strcasecmp(Name, "OSDLeft"))             OSDLeft            = atoi(Value);
@@ -559,6 +561,7 @@ bool cSetup::Save(void)
   Store("UpdateChannels",     UpdateChannels);
   Store("RecordDolbyDigital", RecordDolbyDigital);
   Store("UseDolbyDigital",    UseDolbyDigital);
+  Store("UseSyncEarlyPatch",  UseSyncEarlyPatch);
   Store("ChannelInfoPos",     ChannelInfoPos);
   Store("ChannelInfoTime",    ChannelInfoTime);
   Store("OSDLeft",            OSDLeft);
