@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 1.299 2007/10/06 14:27:18 kls Exp $
+ * $Id: config.h 1.301 2007/11/10 13:38:19 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,15 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.5.10"
-#define VDRVERSNUM   10510  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.5.12"
+#define VDRVERSNUM   10512  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.5.10"
-#define APIVERSNUM   10510  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "1.5.12"
+#define APIVERSNUM   10512  // Version * 10000 + Major * 100 + Minor
+
+#define PLUGINPARAMPATCHVERSNUM 1
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -38,7 +40,7 @@
 
 #define MAINMENUHOOKSVERSNUM 1.0
 
-#define LIEMIKUUTIO  115
+#define LIEMIKUUTIO  116
 
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
@@ -253,6 +255,7 @@ public:
   int UpdateChannels;
   int RecordDolbyDigital;
   int UseDolbyDigital;
+  int UseSyncEarlyPatch;
   int ChannelInfoPos;
   int ChannelInfoTime;
   int OSDLeft, OSDTop, OSDWidth, OSDHeight;
