@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: si.c 1.18 2007/04/22 13:56:39 kls Exp $
+ *   $Id: si.c 1.22 2007/07/21 13:49:48 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -219,7 +219,9 @@ char *String::getText() {
    int len=getLength();
    if (len < 0 || len > 4095)
       return strdup("text error"); // caller will delete it!
-   char *data=new char(len+1);
+   char *data=new char(len+1); // FIXME If this function is ever actually used, this buffer might
+                               // need to be bigger in order to hold the string as UTF-8.
+                               // Maybe decodeText should dynamically handle this? kls 2007-06-10
    decodeText(data, len+1);
    return data;
 }
@@ -248,59 +250,59 @@ char *String::getText(char *buffer, char *shortVersion, int sizeBuffer, int size
 }
 
 static const char *CharacterTables1[] = {
-  NULL,         // 0x00
-  "ISO8859-5",  // 0x01
-  "ISO8859-6",  // 0x02
-  "ISO8859-7",  // 0x03
-  "ISO8859-8",  // 0x04
-  "ISO8859-9",  // 0x05
-  "ISO8859-10", // 0x06
-  "ISO8859-11", // 0x07
-  "ISO8859-12", // 0x08
-  "ISO8859-13", // 0x09
-  "ISO8859-14", // 0x0A
-  "ISO8859-15", // 0x0B
-  NULL,         // 0x0C
-  NULL,         // 0x0D
-  NULL,         // 0x0E
-  NULL,         // 0x0F
-  NULL,         // 0x10
-  "UTF16",      // 0x11
-  "EUC-KR",     // 0x12
-  "GB2312",     // 0x13
-  "GBK",        // 0x14
-  "UTF8",       // 0x15
-  NULL,         // 0x16
-  NULL,         // 0x17
-  NULL,         // 0x18
-  NULL,         // 0x19
-  NULL,         // 0x1A
-  NULL,         // 0x1B
-  NULL,         // 0x1C
-  NULL,         // 0x1D
-  NULL,         // 0x1E
-  NULL,         // 0x1F
+  NULL,          // 0x00
+  "ISO-8859-5",  // 0x01
+  "ISO-8859-6",  // 0x02
+  "ISO-8859-7",  // 0x03
+  "ISO-8859-8",  // 0x04
+  "ISO-8859-9",  // 0x05
+  "ISO-8859-10", // 0x06
+  "ISO-8859-11", // 0x07
+  "ISO-8859-12", // 0x08
+  "ISO-8859-13", // 0x09
+  "ISO-8859-14", // 0x0A
+  "ISO-8859-15", // 0x0B
+  NULL,          // 0x0C
+  NULL,          // 0x0D
+  NULL,          // 0x0E
+  NULL,          // 0x0F
+  NULL,          // 0x10
+  "UTF-16",      // 0x11
+  "EUC-KR",      // 0x12
+  "GB2312",      // 0x13
+  "GBK",         // 0x14
+  "UTF-8",       // 0x15
+  NULL,          // 0x16
+  NULL,          // 0x17
+  NULL,          // 0x18
+  NULL,          // 0x19
+  NULL,          // 0x1A
+  NULL,          // 0x1B
+  NULL,          // 0x1C
+  NULL,          // 0x1D
+  NULL,          // 0x1E
+  NULL,          // 0x1F
 };
 
 #define SingleByteLimit 0x0B
 
 static const char *CharacterTables2[] = {
-  NULL,         // 0x00
-  "ISO8859-1",  // 0x01
-  "ISO8859-2",  // 0x02
-  "ISO8859-3",  // 0x03
-  "ISO8859-4",  // 0x04
-  "ISO8859-5",  // 0x05
-  "ISO8859-6",  // 0x06
-  "ISO8859-7",  // 0x07
-  "ISO8859-8",  // 0x08
-  "ISO8859-9",  // 0x09
-  "ISO8859-10", // 0x0A
-  "ISO8859-11", // 0x0B
-  NULL,         // 0x0C
-  "ISO8859-13", // 0x0D
-  "ISO8859-14", // 0x0E
-  "ISO8859-15", // 0x0F
+  NULL,          // 0x00
+  "ISO-8859-1",  // 0x01
+  "ISO-8859-2",  // 0x02
+  "ISO-8859-3",  // 0x03
+  "ISO-8859-4",  // 0x04
+  "ISO-8859-5",  // 0x05
+  "ISO-8859-6",  // 0x06
+  "ISO-8859-7",  // 0x07
+  "ISO-8859-8",  // 0x08
+  "ISO-8859-9",  // 0x09
+  "ISO-8859-10", // 0x0A
+  "ISO-8859-11", // 0x0B
+  NULL,          // 0x0C
+  "ISO-8859-13", // 0x0D
+  "ISO-8859-14", // 0x0E
+  "ISO-8859-15", // 0x0F
 };
 
 #define NumEntries(Table) (sizeof(Table) / sizeof(char *))
@@ -337,7 +339,7 @@ bool SetSystemCharacterTable(const char *CharacterTable) {
 // default ISO6937 is returned. If a table can be determined, the buffer
 // and length are adjusted accordingly.
 static const char *getCharacterTable(const unsigned char *&buffer, int &length, bool *isSingleByte = NULL) {
-   const char *cs = "ISO6937";
+   const char *cs = "ISO-8859-1";
    if (isSingleByte)
       *isSingleByte = false;
    if (length <= 0)
@@ -370,7 +372,7 @@ static bool convertCharacterTable(const char *from, size_t fromLength, char *to,
 {
   if (SystemCharacterTable) {
      iconv_t cd = iconv_open(SystemCharacterTable, fromCode);
-     if (cd >= 0) {
+     if (cd != (iconv_t)-1) {
         char *fromPtr = (char *)from;
         while (fromLength > 0 && toLength > 1) {
            if (iconv(cd, &fromPtr, &fromLength, &to, &toLength) == size_t(-1)) {
@@ -404,6 +406,9 @@ void String::decodeText(char *buffer, int size) {
       }
    bool singleByte;
    const char *cs = getCharacterTable(from, len, &singleByte);
+   // FIXME Need to make this UTF-8 aware (different control codes).
+   // However, there's yet to be found a broadcaster that actually
+   // uses UTF-8 for the SI data... (kls 2007-06-10)
    for (int i = 0; i < len; i++) {
       if (*from == 0)
          break;
@@ -440,6 +445,9 @@ void String::decodeText(char *buffer, char *shortVersion, int sizeBuffer, int si
       }
    bool singleByte;
    const char *cs = getCharacterTable(from, len, &singleByte);
+   // FIXME Need to make this UTF-8 aware (different control codes).
+   // However, there's yet to be found a broadcaster that actually
+   // uses UTF-8 for the SI data... (kls 2007-06-10)
    for (int i = 0; i < len; i++) {
       if (    ((' ' <= *from) && (*from <= '~'))
            || (*from == '\n')
